@@ -1,5 +1,6 @@
 /*
- * Copyright (C)2011, 2013, 2018, 2023 D. R. Commander.  All Rights Reserved.
+ * Copyright (C)2011, 2013, 2018, 2022-2023 D. R. Commander.
+ *                                          All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -95,29 +96,30 @@ public class TJTransform extends Rectangle {
    * TJTransformer.transform()} to throw an exception if the transform is not
    * perfect.  Lossless transforms operate on MCU blocks, whose size depends on
    * the level of chrominance subsampling used.  If the image's width or height
-   * is not evenly divisible by the MCU block size (see {@link TJ#getMCUWidth}
-   * and {@link TJ#getMCUHeight}), then there will be partial MCU blocks on the
-   * right and/or bottom edges.  It is not possible to move these partial MCU
-   * blocks to the top or left of the image, so any transform that would
-   * require that is "imperfect."  If this option is not specified, then any
-   * partial MCU blocks that cannot be transformed will be left in place, which
-   * will create odd-looking strips on the right or bottom edge of the image.
+   * is not evenly divisible by the MCU block size (see {@link TJ#getMCUWidth
+   * TJ.getMCUWidth()} and {@link TJ#getMCUHeight TJ.getMCUHeight()}), then
+   * there will be partial MCU blocks on the right and/or bottom edges.  It is
+   * not possible to move these partial MCU blocks to the top or left of the
+   * image, so any transform that would require that is "imperfect."  If this
+   * option is not specified, then any partial MCU blocks that cannot be
+   * transformed will be left in place, which will create odd-looking strips on
+   * the right or bottom edge of the image.
    */
-  public static final int OPT_PERFECT     = 1;
+  public static final int OPT_PERFECT     = (1 << 0);
   /**
    * This option will discard any partial MCU blocks that cannot be
    * transformed.
    */
-  public static final int OPT_TRIM        = 2;
+  public static final int OPT_TRIM        = (1 << 1);
   /**
    * This option will enable lossless cropping.
    */
-  public static final int OPT_CROP        = 4;
+  public static final int OPT_CROP        = (1 << 2);
   /**
    * This option will discard the color data in the source image and produce a
    * grayscale destination image.
    */
-  public static final int OPT_GRAY        = 8;
+  public static final int OPT_GRAY        = (1 << 3);
   /**
    * This option will prevent {@link TJTransformer#transform
    * TJTransformer.transform()} from outputting a JPEG image for this
@@ -125,20 +127,36 @@ public class TJTransform extends Rectangle {
    * filter to capture the transformed DCT coefficients without transcoding
    * them.
    */
-  public static final int OPT_NOOUTPUT    = 16;
+  public static final int OPT_NOOUTPUT    = (1 << 4);
   /**
    * This option will enable progressive entropy coding in the JPEG image
    * generated by this particular transform.  Progressive entropy coding will
    * generally improve compression relative to baseline entropy coding (the
    * default), but it will reduce decompression performance considerably.
+   * Can be combined with {@link #OPT_ARITHMETIC}.  Implies
+   * {@link #OPT_OPTIMIZE} unless {@link #OPT_ARITHMETIC} is also specified.
    */
-  public static final int OPT_PROGRESSIVE = 32;
+  public static final int OPT_PROGRESSIVE = (1 << 5);
   /**
    * This option will prevent {@link TJTransformer#transform
    * TJTransformer.transform()} from copying any extra markers (including EXIF
    * and ICC profile data) from the source image to the destination image.
    */
-  public static final int OPT_COPYNONE    = 64;
+  public static final int OPT_COPYNONE    = (1 << 6);
+  /**
+   * This option will enable arithmetic entropy coding in the JPEG image
+   * generated by this particular transform.  Arithmetic entropy coding will
+   * generally improve compression relative to Huffman entropy coding (the
+   * default), but it will reduce decompression performance considerably.  Can
+   * be combined with {@link #OPT_PROGRESSIVE}.
+   */
+  public static final int OPT_ARITHMETIC  = (1 << 7);
+  /**
+   * This option will enable optimized baseline entropy coding in the JPEG
+   * image generated by this particular transform.  Optimized baseline entropy
+   * coding will improve compression slightly (generally 5% or less.)
+   */
+  public static final int OPT_OPTIMIZE    = (1 << 8);
 
 
   /**
@@ -151,10 +169,12 @@ public class TJTransform extends Rectangle {
    * Create a new lossless transform instance with the given parameters.
    *
    * @param x the left boundary of the cropping region.  This must be evenly
-   * divisible by the MCU block width (see {@link TJ#getMCUWidth})
+   * divisible by the MCU block width (see {@link TJ#getMCUWidth
+   * TJ.getMCUWidth()})
    *
    * @param y the upper boundary of the cropping region.  This must be evenly
-   * divisible by the MCU block height (see {@link TJ#getMCUHeight})
+   * divisible by the MCU block height (see {@link TJ#getMCUHeight
+   * TJ.getMCUHeight()})
    *
    * @param w the width of the cropping region.  Setting this to 0 is the
    * equivalent of setting it to (width of the source JPEG image -
@@ -169,8 +189,8 @@ public class TJTransform extends Rectangle {
    * @param options the bitwise OR of one or more of the transform options
    * ({@link #OPT_PERFECT OPT_*})
    *
-   * @param cf an instance of an object that implements the {@link
-   * TJCustomFilter} interface, or null if no custom filter is needed
+   * @param cf an instance of an object that implements the
+   * {@link TJCustomFilter} interface, or null if no custom filter is needed
    */
   @SuppressWarnings("checkstyle:HiddenField")
   public TJTransform(int x, int y, int w, int h, int op, int options,
@@ -184,18 +204,18 @@ public class TJTransform extends Rectangle {
   /**
    * Create a new lossless transform instance with the given parameters.
    *
-   * @param r a <code>Rectangle</code> instance that specifies the cropping
-   * region.  See {@link
-   * #TJTransform(int, int, int, int, int, int, TJCustomFilter)} for more
-   * detail.
+   * @param r a <code>java.awt.Rectangle</code> instance that specifies the
+   * cropping region.  See
+   * {@link #TJTransform(int, int, int, int, int, int, TJCustomFilter)} for
+   * more details.
    *
    * @param op one of the transform operations ({@link #OP_NONE OP_*})
    *
    * @param options the bitwise OR of one or more of the transform options
    * ({@link #OPT_PERFECT OPT_*})
    *
-   * @param cf an instance of an object that implements the {@link
-   * TJCustomFilter} interface, or null if no custom filter is needed
+   * @param cf an instance of an object that implements the
+   * {@link TJCustomFilter} interface, or null if no custom filter is needed
    */
   @SuppressWarnings("checkstyle:HiddenField")
   public TJTransform(Rectangle r, int op, int options,
